package de.moritzruth.dracula_musical.act

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.dracula_musical.song.*
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.PropPosition
import de.moritzruth.theaterdsl.show.ShowBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.seconds

fun ShowBuilderContext.act1() = act("Erster Akt") {
    scene("Ouvertüre") {
        step(StepCue.MusicStart("Ouvertüre", 10.seconds)) {
            curtainState = CurtainState.CLOSED

            Washs.both {
                it.pointAtAudience()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
                it.beamAngle.static(100.percent)
            }
        }

        step(StepCue.MusicEnd) {
            actors {
                +"Erzähler / in der Mitte der Vorbühne"
            }

            rightSpotTarget = "Erzähler / Mitte der Vorbühne"

            onRun {
                Spots.right.brightness.fade(100.percent, 2.seconds)
            }
        }

        step(StepCue.Text("Erzähler", "Der Mittsommernacht.")) {
            actors {
                -"Erzähler"
            }

            rightSpotTarget = null

            onRun {
                Spots.right.brightness.off(1.seconds)
            }
        }
    }

    scene("Mittsommernacht") {
        songMittsommernacht()
    }

    scene("Ankunft in Huntington") {
        step(StepCue.Text("Mina", "Das ist also Huntington!")) {
            sidelight.brightness.fade(75.percent, 15.seconds)
            sidelight.color1.fadeRandomAround(130.degrees, 5.degrees, 10.seconds)
            sidelight.color2.fadeRandomAround(60.degrees, 5.degrees, 9.seconds)
            sidelight.color3.fadeRandomAround(130.degrees, 5.degrees, 12.seconds)
            sidelight.color4.fadeRandomAround(60.degrees, 5.degrees, 11.seconds)
        }
    }

    scene("Hawkins’ Feierabend") {
        step(StepCue.Text("Lucy", "Heute gehört Mister Harker uns.", "Ende")) {
            actors {
                -"Mina"
                -"Lucy"
                -"Jonathan"
                +"Hawkins"
                // TODO: Expand
                +"Sekretärinnen"
            }

            props {
                it[PropPosition.CENTER] = "Hawkins’ Stuhl"
            }
        }

        step(StepCue.Text("Sekretärin 1", "Ashton und Ashton 6 Pfund glatt…")) {
            actors {
                +"Renfield"
            }
        }

        songAdelIstGeil()

        step(StepCue.Text("Renfield", "Ich bin schon weg.")) {
            actors {
                -"Renfield"
            }
        }

        step(StepCue.MusicStart("Umbau (Adel ist geil)", 10.seconds)) {
            actors {
                -"Hawkins"
                -"Sekretärinnen"
            }

            curtainState = CurtainState.CLOSED

            props {
                it[PropPosition.CENTER] = null
            }

            onRun {
                FrontLights.all { it.brightness.off(6.seconds) }
                sidelight.brightness.off(6.seconds)
            }
        }
    }

    scene("Die Irrenanstalt") {
        step(StepCue.MusicEnd) {
            actors {
                +"Ascot"
                +"Mina"
            }

            curtainState = CurtainState.OPEN

            onRun {
                FrontLights.all { it.brightness.fade(40.percent, 6.seconds) }
                sidelight.brightness.fade(100.percent, 6.seconds)
                sidelight.colors.forEach { it.static(Color(hue = 180.degrees, saturation = 10.percent)) }
                backlightBar.brightness.fade(50.percent, 6.seconds)
                backlightBar.color.static(Color(hue = 280.degrees, saturation = 50.percent))
            }
        }

        step(StepCue.Text("Ascot", "Da kommt der Rest der Familie!")) {
            actors {
                +"Oberschwester"
                // TODO: Expand
                +"Pflegeteam"
            }
        }

        songIrrenhaus()

        step(StepCue.Text("Oberschwester", "Sonst kriegst du sie als Zäpfchen!")) {
            actors {
                -"Oberschwester"
                -"Pflegeteam"
                -"Die Irren"
                +"Renfield"
                +"Vermieterin"
                +"Wärter 1"
                +"Wärter 2"
            }
        }

        step(StepCue.Text("Dr. Sewart", "Ich liebe meinen Job!")) {
            actors {
                -"Renfield"
                -"Vermieterin"
                -"Wärter 1"
                -"Wärter 2"
                -"Dr. Sewart"
                -"Ascot"
            }
        }

        step(StepCue.Custom("Mina öffnet die Tür")) {
            actors {
                +"Mr. Nickelby"
                +"Mrs. Nickelby"
                +"Tochter Nickelby"
            }
        }

        step(StepCue.Text("Mina", "Es gibt hier noch mehr Irre.")) {
            actors {
                -"Mina"
            }
        }
    }

    scene("Blockflötenkreis") {
        songLucy()

        lightStep(StepCue.Custom("Gardine wird geöffnet")) {
            FrontLights.all { it.brightness.fade(100.percent, 2.seconds) }
            sidelight.brightness.off(3.seconds)
        }

        step(StepCue.Text("Mr. Nickelby", "…als du die Vorhänge aufgemacht hast.")) {
            actors {
                +"Ascot"
            }

            onRun {
                sidelight.colors.forEach { it.fadeRandomAround(0.degrees, 5.degrees, 3.seconds) }
            }
        }

        step(StepCue.Text("Ascot", "Sie Flötenguru!")) {
            actors {
                +"Dr. Sewart"
            }
        }

        repeat(2) {
            lightStep(StepCue.Custom("Gardine wird geschlossen")) {
                FrontLights.all { it.brightness.fade(40.percent, 2.seconds) }
                sidelight.brightness.fade(80.percent, 2.seconds)
            }

            lightStep(StepCue.Custom("Gardine wird geöffnet")) {
                FrontLights.all { it.brightness.fade(100.percent, 2.seconds) }
                sidelight.brightness.off(2.seconds)
            }
        }

        step(StepCue.Text("Dr. Sewart", "Und dass mir ja die Gardine aufbleibt!")) {
            actors {
                -"Dr. Sewart"
                -"Ascot"
                -"Lucy"
                -"Mr. Nickelby"
                -"Mrs. Nickelby"
                -"Tochter Nickelby"
            }

            onRun {
                FrontLights.all { it.brightness.off(5.seconds) }
            }
        }
    }

    scene("Männerbesuch") {
        step(StepCue.Custom("Es klingelt")) {
            actors {
                +"Mina"
                +"Jonathan / hinter der Tür"
            }

            onRun {
                FrontLights.all { it.brightness.fade(75.percent, 5.seconds) }
            }
        }

        step(StepCue.Text("Mina", "Sehr merkwürdig.")) {
            actors {
                +"Ascot"
            }
        }

        step(StepCue.Text("Ascot", "Miss Murray!", "beim dritten Mal")) {
            actors {
                -"Ascot"
            }
        }

        songDuett()

        step(StepCue.Text("Mina", "So markiere ich, was mir gehört.")) {
            actors {
                +"Ascot"
            }
        }

        step(StepCue.Text("Jonathan", "Gute Nacht.")) {
            actors {
                -"Mina"
                -"Jonathan"
                -"Ascot"
            }

            leftSpotTarget = null
            curtainState = CurtainState.CLOSED

            onRun {
                FrontLights.all { it.brightness.off(5.seconds) }
            }
        }

        step(StepCue.MusicStart("Umbau (Duett)", 10.seconds)) {}
    }

    scene("Kaffeeklatsch") {
        step(StepCue.MusicEnd) {
            actors {
                +"Dr. Sewart"
            }

            curtainState = CurtainState.OPEN
            rightSpotTarget = "Dr. Sewart"

            onRun {
                sidelight.colors.forEach { it.static(Color(hue = 180.degrees, saturation = 10.percent)) }
                sidelight.brightness.fade(10.percent, 6.seconds)
            }
        }

        step(StepCue.Custom("Dr. Sewart ist versteckt")) {
            actors {
                +"Oberschwester"
            }

            onRun {
                FrontLights.all { it.brightness.fade(50.percent, 3.5.seconds) }
                sidelight.brightness.fade(75.percent, 4.seconds)
                Spots.right.brightness.static(100.percent)
            }
        }

        step(StepCue.Text("Dr. Sewart", "Da sind sie!")) {
            actors {
                +"Ascot"
                // TODO: Expand
                +"Kaffeetanten"
                // TODO: Expand
                +"Nichten"
            }

            onRun {
                FrontLights.all { it.brightness.fade(100.percent, 3.seconds) }
                sidelight.brightness.off(10.seconds)
            }
        }

        step(StepCue.Text("Dr. Sewart", "Ich mache Diät!")) {
            actors {
                -"Dr. Sewart"
                -"Oberschwester"
            }
        }

        songKaffeeklatsch()
    }

    scene("Ankunft Van Helsings") {
        step(StepCue.Text("Ascot", "Jetzt ist Lucy wieder ganz gesund.")) {
            actors {
                +"Dr. Sewart"
            }

            onRun {
                Washs.both {
                    it.pointAtCeiling()
                    it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
                    it.magenta.reset()
                }
            }
        }

        step(StepCue.Text("Dr. Sewart", "Was mach ich nur!")) {
            actors {
                +"Van Helsing"
            }

            rightSpotTarget = "Van Helsing"

            onRun {
                Spots.right.brightness.static(100.percent)
                FrontLights.all { it.brightness.pulseOnce(
                    rampUpDuration = 1.seconds,
                    rampDownDuration = 10.seconds,
                    peak = 0.percent,
                    start = 100.percent,
                    end = 100.percent
                ) }

                BlinderBars.all {
                    it.preset.static(null)
                    it.color.static(Color.WHITE)
                    it.white.static(40.percent)
                    it.brightness.pulseOnce(0.3.seconds, 2.seconds, 100.percent)
                }

                Washs.both {
                    it.pointAtAudience()
                    it.beamAngle.fade(0.percent, 2.seconds)
                    it.brightness.fade(start = 100.percent, end = 0.percent, duration = 2.seconds)
                }
            }
        }

        step(StepCue.Text("Van Helsing", "Oder will jemand von den Damen?")) {
            actors {
                -"Kaffeetanten"
                -"Nichten"
            }
        }

        step(StepCue.Text("Van Helsing", "Helfen Sie mir jetzt!")) {
            actors {
                -"Dr. Sewart"
                -"Van Helsing"
                -"Ascot"
            }

            curtainState = CurtainState.CLOSED

            onRun {
                FrontLights.all { it.brightness.off(3.seconds) }

                BlinderBars.all {
                    it.color.reset()
                    it.white.reset()
                }
            }
        }
    }

    scene("Verträge") {
        step(StepCue.Custom("Vorhang ist geschlossen")) {
            actors {
                +"Hawkins / durchs Publikum"
                +"Renfield / durchs Publikum"
            }

            rightSpotTarget = "Hawkins & Renfield (im Publikum)"

            onRun {
                Spots.right.brightness.fade(100.percent, 3.seconds)
            }
        }

        step(StepCue.Text("Hawkins", "…zum Grafen Dracula!")) {
            actors {
                -"Hawkins / durch den Vorhang"
                -"Renfield / durch den Vorhang"
            }

            rightSpotTarget = "mittig verbleiben"

            onRun {
                Spots.right.brightness.off(10.seconds)
            }
        }
    }

    scene("Einbruch") {
        step(StepCue.Custom("Umbau abgeschlossen")) {
            actors {
                +"Backe / von links"
                +"Timmy / von links"
                +"Eloïse / im Sarg"
                +"Françoise / im Sarg"
                +"Bernadette / im Sarg"
            }

            curtainState = CurtainState.OPEN

            onRun {
                sidelight.color1.fadeRandomAround(230.degrees, 10.degrees, 10.seconds)
                sidelight.color2.static(Color(hue = 230.degrees, saturation = 60.percent))
                sidelight.color3.static(Color(hue = 230.degrees, saturation = 10.percent))
                sidelight.color4.static(Color(hue = 230.degrees, saturation = 10.percent))
                sidelight.brightness.fade(40.percent, 8.seconds)
            }
        }

        songEsIstAngerichtet()
        songDraculasZorn()
    }
}